/*

	DiVAFilter.h
	
	Interface definitons and specifications for DiVA VideoShop filter
	effect code modules.
	
	09/16/91	Ivn Cavero Belande
	
	 1991,1992 DiVA Corporation. All rights reserved.
	
*/

#ifndef DIVAFILTER
#define DIVAFILTER

#include <Types.h>
#include <QuickDraw.h>
#include <QDOffscreen.h>

/*	Selector constants	*/

#define FiltParameters		0
	/*	Initialize and display paramters dialog		*/
#define FiltPrepare			1
	/*	Prepare for applying filter					*/
#define	FiltStart			2
	/*	Initialize filter-wide globals and buffers	*/
#define FiltProcessFrame	3
	/*	Apply filter to frame						*/
#define FiltFinish			4
	/*	Deallocate globals							*/

#define	kFiltVersion	1
	/*	Filter interface version		*/
#define	kFiltCodeType	'Vflt'
	/*	Type of filter code resource	*/
#define kFiltInfoType	'FltI'
	/*	Type of filter info resource	*/

enum {
	FiltErrReported				=	0x00010000,		/*	An error has occurred. Do not display dialog.		*/
	FiltErrOutOfDiskSpace		=	0x00030000,		/*	"The effect could not be applied because there
														is not enough disk space in the recording bin."		*/
	FiltErrOutOfMemory			=	0x00040000,		/*	"The effect could not be applied because there
														is not enough memory available."					*/
	FiltErrEffectFileCorrupted	=	0x00050000,		/*	"The effect module is corrupted. Please replace
														it with the original effects module."				*/
	FiltErrBadSystemVersion		=	0x00060000,		/*	"This effect module will not function with the
														currently installed system software."				*/
	FiltErrBadQDVersion			=	0x00080000,		/*	"This effect module will not work with the currently
														installed version of 32-bit QuickDraw."				*/
	FiltErrBadQTVersion			=	0x000A0000,		/*	"This effect module will not work with the currently
														installed version of QuickTime."					*/
	FiltErrBadVSVersion			=	0x000C0000,		/*	"This effect module will not work with this version
														of DiVA VideoShop."								*/
};

enum {
	filterFlagTimeBased = 1<<0	/*	Filter is dynamic. Feed frames at specific interfals whether
										they are different frames or not.	*/
};

typedef struct FiltRecord {
	long		version;		/*	Version of Filter Interface used by called. This version
									is version 1.													*/
	ProcPtr 	abortProc;		/*	Pointer to no-argument BOOLEAN pascal function that returns
									whether the user has canceled the filtering operation.			*/
	ProcPtr 	progressProc;	/*	Pointer to two-argument pascal procedure that accepts two
									LONGINTs. The first one describes how much of the filtering
									operation for this frame has been completed. The second one
									describes the number of operations for this frame.				*/
	Handle		parameters;		/*	Plug-in's storage handle, initialized to NIL by VideoShop
									unless passing same parameters handle as an earlier filtering
									operation. Must be self-contained, and must not be disposed
									of by the filter.												*/
	long		timeScale;		/*	The time unit of other time measurements, such as frame and
									timeStep, in ticks per second.									*/
	long		timeStep;		/*	The amount of ticks stepped forward in the sequence between
									successive calls; valid only for dynamic filters.				*/
	RGBColor	background;		/*	The background color. If the filtering operation leaves any
									part of the image "empty," it should be filled with this color.	*/
	PixMapHandle	srcImage;	/*	The original source image, a standard QuickDraw PixMapHandle,
									not-purgeable, and unlocked.									*/
	GWorldPtr		destImage;	/*	The destination image buffer for the filterd image.				*/
}	FiltParams, *FiltParamPtr;

typedef struct FiltInfoRec {
	long	version;			/*	Version number of VideoShop's calling interface this filter is
									compatible with.													*/
	OSType	media;				/*	Media this filter operate on. Currently only filters that operate
									on video media ('vide') are supported.								*/
	long	flags;				/*	Filter flags. Currently only filterFlagTimeBased is defined.		*/
	OSType	filtSignature;		/*	Filter signature. A unique 4-character ID for this filter.
									Analogous to a QuickTime component componentSubType.				*/
	long	filtVersion;		/*	Filter version.														*/
	OSType	filtManufacturer;	/*	Filter manufacturer. A 4-character ID for the manufacturer of this
									filter, analogous, to a QuickTime component componentManufacturer.	*/
	OSType	descRsrcType;		/*	Resource type and ID of description of filter. Description should
									be a pascal string.													*/
	short	descRsrcID;
/*	Reserved for future expansion	*/
}	**FilterInfo;

typedef unsigned long FiltErr;

typedef pascal FiltErr (*FiltPtr)(short selector, FiltParamPtr filterRecord,
	long *data, short currentStep, short totalSteps);

#endif DIVAFILTER
